//
// @file:			applet.cc
// author:			E^rthW( )rm 2003
// last mod:			Friday, September 05 2003
// about:			defines the c_applet class
// license:			GPL

// includes //
//
#include "applet.hh"
#include <config.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>
#include <gtkmm/main.h>
#include <glib/gi18n.h>
#include <libgnomeuimm.h>
#include "config.h"

// definitions //
//

void 
display_forward				(BonoboUIComponent		*, 
					 void				* p_applet,
					 const gchar			*)
{
	static_cast<c_applet *>(p_applet)->on_forward_activated();
}// end display_preferences

void 
display_search				(BonoboUIComponent		*, 
					 void				* p_applet,
					 const gchar			*)
{
	static_cast<c_applet *>(p_applet)->on_search_activated();
}// end display_preferences

void 
display_delete				(BonoboUIComponent		*, 
					 void				* p_applet,
					 const gchar			*)
{
	static_cast<c_applet *>(p_applet)->on_delete_activated();
}// end display_preferences

void 
display_preferences			(BonoboUIComponent		*, 
					 void				* p_applet,
					 const gchar			*)
{
	static_cast<c_applet *>(p_applet)->on_preferences_activated();
}// end display_preferences

void
display_help				(BonoboUIComponent		*, 
					 void 				* p_applet,
					 const gchar			*)
{
	static_cast<c_applet *>(p_applet)->on_help_activated();
}// end display_help

void
display_about				(BonoboUIComponent		*,
					 void				* p_applet,
					 const gchar			*)
{
	static_cast<c_applet *>(p_applet)->on_about_activated();
}// end display_about

// c_applet::c_applet //
//
c_applet::c_applet			(PanelApplet			* p_pa) :
	m_p_panel_applet(p_pa),
	m_p_preferences_dialog(NULL),
	m_p_search_dialog(NULL)
{
	// set up drag and drop
	// targets:
	std::list<Gtk::TargetEntry> ls_drop_target;
	ls_drop_target.push_back(Gtk::TargetEntry("STRING"));
	ls_drop_target.push_back(Gtk::TargetEntry("image/jpeg"));

	// make this a dnd drop destination
	drag_dest_set(ls_drop_target, Gtk::DEST_DEFAULT_ALL, Gdk::DragAction(GDK_ACTION_COPY | GDK_ACTION_MOVE));

	// connect dnd signals
	signal_drag_data_received().connect(sigc::mem_fun(*this, &c_applet::on_drop_drag_data_received));

	// set up the gconf client
	m_ref_gconf_client = Gnome::Conf::Client::get_default_client();

	m_ref_gconf_client->add_dir("/apps/wp_tray");

	// add our directory to the list of directories the GConfClient will watch
	m_ref_gconf_client->add_dir("/desktop/gnome/background");

	// listen to changes to our key
	m_n_cnxn_update_tooltip = 
	m_ref_gconf_client->notify_add(	"/desktop/gnome/background/picture_filename",
					sigc::mem_fun(*this, &c_applet::create_tooltip));

	// listen to changes to our key
	m_n_cnxn_render_thumb =
	m_ref_gconf_client->notify_add(	"/desktop/gnome/background/picture_filename",
					sigc::mem_fun(*this, &c_applet::render_thumbnail));


	// create app icon (for help dialog)
	try
	{
		m_ref_gdk_icon = Gdk::Pixbuf::create_from_file(GNOMEICONDIR"/wp_tray-applet.png");
	}// end try
	catch(...)
	{
		// it's a minor problem if we can't find the icon
		m_ref_gdk_icon = Glib::RefPtr<Gdk::Pixbuf>();
	}// end catch

	m_gtk_tooltips.enable();

	// create icon for the panel from the current wp
	c_applet::render_thumbnail(	m_n_cnxn_render_thumb,
					m_ref_gconf_client->get_entry("/desktop/gnome/background/picture_filename"));

	c_applet::create_tooltip(	m_n_cnxn_update_tooltip,
					m_ref_gconf_client->get_entry("/desktop/gnome/background/picture_filename"));

	// show the panel image
	c_applet::add(m_gtk_image);
	m_gtk_image.show_all();
	

	// setup menu
	static const char str_menu_xml[] =
		"<popup name=\"button3\">\n"
		"   <menuitem name=\"Forward Item\"\n"
		"             verb=\"WpTrayForward\" _label=\"_Forward\"\n"
		"             pixtype=\"stock\" pixname=\"gtk-go-forward\"/>\n"
		"   <menuitem name=\"Search Item\"\n"
		"             verb=\"WpTraySearch\" _label=\"_Search\"\n"
		"             pixtype=\"stock\" pixname=\"gtk-find\"/>\n"
		"   <menuitem name=\"Delete Item\"\n"
		"             verb=\"WpTrayDelete\" _label=\"_Delete\"\n"
		"             pixtype=\"stock\" pixname=\"gtk-delete\"/>\n"
		"   <menuitem name=\"Properties Item\"\n"
		"             verb=\"WpTrayPreferences\" _label=\"_Preferences\"\n"
		"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
		"   <menuitem name=\"Help Item\"\n"
		"             verb=\"WpTrayHelp\" _label=\"_Help\"\n"
		"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
		"   <menuitem name=\"About Item\"\n"
		"             verb=\"WpTrayAbout\" _label=\"_About\"\n"
		"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
		"</popup>\n";

	static const BonoboUIVerb ar_menu_verbs[] =
	{
		BONOBO_UI_VERB("WpTrayForward", &display_forward),
		BONOBO_UI_VERB("WpTraySearch", &display_search),
		BONOBO_UI_VERB("WpTrayDelete", &display_delete),
		BONOBO_UI_VERB("WpTrayPreferences", &display_preferences),
		BONOBO_UI_VERB("WpTrayHelp", &display_help),
		BONOBO_UI_VERB("WpTrayAbout", &display_about),    
		BONOBO_UI_VERB_END
  	};

	panel_applet_setup_menu(m_p_panel_applet, str_menu_xml, ar_menu_verbs, this);
}// end constructor

// c_applet:::~c_applet //
//
c_applet::~c_applet			()
{
	if(m_p_preferences_dialog != NULL)
	{
		delete m_p_preferences_dialog;
	}// end if
}// end destructor

// c_applet::c_applet //
//
Gtk::Container &
c_applet::get_container			()
{
  return *this;
}

// c_applet::get_size //
//
int
c_applet::get_size			() const
{
	// FIXME: the panel appears to lie about its true size
	// 2 pixels of frame decoration on both sides
	return panel_applet_get_size(m_p_panel_applet) - 2 * 2;
}

// c_applet::horizontal //
//
bool
c_applet::horizontal			() const
{
	PanelAppletOrient orient = panel_applet_get_orient(m_p_panel_applet);
	return orient == PANEL_APPLET_ORIENT_UP || orient == PANEL_APPLET_ORIENT_DOWN;
}

// c_applet::on_forward_activated //
//
void
c_applet::on_forward_activated		()
{
	m_wallpapers.set_random();
}// end c_applet::on_forward_activated

// c_applet::on_search_activated //
//
void
c_applet::on_search_activated	()
{
	if(m_p_search_dialog == NULL)
	{
		// load the Glade file and instiate its widgets:
		Glib::RefPtr<Gnome::Glade::Xml> ref_xml;

		try
		{
			ref_xml = Gnome::Glade::Xml::create(WP_TRAY_GLADEDIR"wp_tray_search.glade");
		}// end try
		catch(const Gnome::Glade::XmlError & ex)
		{
			std::cerr << ex.what() << std::endl;
		}// end catch

		// get the glade-instantiated dialog
		ref_xml->get_widget_derived("window1", m_p_search_dialog);
	}// end if

	m_p_search_dialog->show();
}// end c_applet::on_search_activated


// c_applet::on_delete_activated //
//
void
c_applet::on_delete_activated		()
{
	Gtk::MessageDialog gtk_msg_dlg(_("Are you sure you want to delete this wallpaper from disk?"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);

	int n_result = gtk_msg_dlg.run();
	
	switch(n_result)
	{
		case Gtk::RESPONSE_YES:
		{
			m_wallpapers.delete_curret();
		}// end case
		break;
		
		default:
		{
			// do i care? no.
		}// end default
		break;
	}// end switch
}// end c_applet::on_delete_activated

// c_applet::on_preferences_activated //
//
void
c_applet::on_preferences_activated	()
{
	if(m_p_preferences_dialog == NULL)
	{
		// load the Glade file and instiate its widgets:
		Glib::RefPtr<Gnome::Glade::Xml> ref_xml;

		try
		{
			ref_xml = Gnome::Glade::Xml::create(WP_TRAY_GLADEDIR"wp_tray_conf.glade");
		}// end try
		catch(const Gnome::Glade::XmlError & ex)
		{
			std::cerr << ex.what() << std::endl;
		}// end catch

		// get the glade-instantiated dialog
		ref_xml->get_widget_derived("window1", m_p_preferences_dialog);
	}// end if

	m_p_preferences_dialog->show();
}// end c_applet::on_preferences_activated

// c_applet::on_help_activated //
//
void
c_applet::on_help_activated		()
{
}// end c_applet::on_help_activated

// c_applet::on_about_activated //
//
void
c_applet::on_about_activated		()
{
	std::vector<Glib::ustring> authors;
	authors.push_back("E^r7hW( )rm <earthworm@planetearthworm.com>");
  
	std::vector<Glib::ustring> documenters;
	// add documenters here

	// note to translators: please fill in your names and email addresses
	Glib::ustring translators = _("translator-credits");

	Glib::ustring description =
		_("Wallpaper Tray is a wallpaper utility that sits on your GNOME panel. "
		"It gives you a random wallpaper from a chosen directory at logon, on a timer, and allows you to "
		"select a new wallpaper at random from its menu.");
  
	if(m_p_about_box.get() == 0)
	{
		m_p_about_box.reset
			(
			new Gnome::UI::About
				(
				_("Wallpaper Tray"), VERSION,
				_("Copyleft © 2005 E^r7hW( )rm"),
				authors, documenters, description,
				translators == "translator-credits"
				? "" : translators,
				m_ref_gdk_icon)
			);

		m_p_about_box->set_icon(m_ref_gdk_icon);
		m_p_about_box->show();
	}
	else
	{
		m_p_about_box->show();
		m_p_about_box->raise();
	}// end else
}// end c_applet::on_about_activated

void
c_applet::on_drop_drag_data_received	(const Glib::RefPtr<Gdk::DragContext>& context,
					 int				n_x,
					 int				n_y,
					 const Gtk::SelectionData	& selection_data,
					 guint				,
					 guint				n_time)
{
	Gtk::MessageDialog gtk_msg_dlg(_("Drag and drop not implemented yet."), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);

	gtk_msg_dlg.run();
 
	//if((selection_data.get_length() >= 0) && (selection_data.get_format() == 8))
	//{
	//	std::cout << "Received \"" << selection_data.get_data_as_string() << "\" in label " << std::endl;
	//}// end if

	context->drag_finish(false, false, n_time);
}// end c_applet::on_drop_drag_data_received

void
c_applet::render_thumbnail		(guint				n_cnxn,
					 Gnome::Conf::Entry		gconf_entry)
{
	Glib::RefPtr<Gdk::Pixbuf> gdk_pixbuf = Glib::RefPtr<Gdk::Pixbuf>();

	int targetw, targeth;
	if (c_applet::horizontal()) {
	    targetw = (int)(c_applet::get_size() * 1.3);
	    targeth = c_applet::get_size();
	} else {
	    targetw = c_applet::get_size();
	    targeth = (int)(c_applet::get_size() * 0.77);
	}

	try
	{
		gdk_pixbuf = 
		Gdk::Pixbuf::create_from_file(m_ref_gconf_client->get_string("/desktop/gnome/background/picture_filename"));
	}// end try
	catch(...)
	{
		// bugger, slap a black square in there for now
	    gdk_pixbuf =
		Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, 0, 8, targetw,
				    targeth);

		gdk_pixbuf->fill(0x000000ff);

		m_gtk_image.set(gdk_pixbuf);

		return;
	}// end catch

	Glib::RefPtr<Gdk::Pixbuf> gdk_pixbuf_scaled = Glib::RefPtr<Gdk::Pixbuf>();

	// Only scale down if we're sure the starting image is
	// completely larger than the destination size (or it will
	// abort).
	if (gdk_pixbuf->get_width() < targetw ||
	    gdk_pixbuf->get_height() < targeth) {
	    gdk_pixbuf_scaled = gdk_pixbuf;
	} else {

	    gdk_pixbuf_scaled = 
		Gnome::UI::thumbnail_scale_down_pixbuf(gdk_pixbuf,
						       targetw,
						       targeth);
	}

	m_gtk_image.set(gdk_pixbuf_scaled);
}// end c_applet::render_thumbnail

void
c_applet::create_tooltip		(guint				n_cnxn,
					 Gnome::Conf::Entry		gconf_entry)
{
	std::ostringstream ss;
	ss.imbue(std::locale("")); // use the user's locale for this stream

	// create tip text
	ss << _("Wallpaper Tray") << " ";
	ss << VERSION << " ";
	ss << m_ref_gconf_client->get_string("/desktop/gnome/background/picture_filename");

	// attach it
	m_gtk_tooltips.set_tip(*this, Glib::locale_to_utf8(ss.str()));
}// end c_applet::create_tooltip
