//
// file:	file.cc
// author:	earthworm
// date:	Sunday, 01 May 2005
// about:	implements preferences dialog for wp_tray
//

// includes //
//
#include "preferences_dialog.hh"
#include "config.h"
#include <glib/gi18n.h>
#include <stdexcept>
#include <sstream>
#include <algorithm>

// definitions //
//
c_preferences_dialog::c_preferences_dialog	(BaseObjectType				* p_window,
						 const Glib::RefPtr<Gnome::Glade::Xml>	& ref_glade) :
	Gtk::Dialog(p_window),
	m_ref_glade(ref_glade),
	m_p_button_close(NULL),
	m_p_button_apply(NULL),
	m_p_button_add_dir(NULL),
	m_p_button_rem_dir(NULL),
	m_p_entry_time(NULL),
	m_p_entry_mode(NULL),
	m_p_entry_selection_mode(NULL),
	m_p_checkbutton_timeout(NULL),
	m_p_checkbutton_logon_new(NULL),
	m_p_checkbutton_image_check(NULL),
	m_p_checkbutton_follow_links(NULL)
{
	// set up the gconf client
	m_ref_gconf_client = Gnome::Conf::Client::get_default_client();

	// get the glade instantiated widgets and connect signals
	//
	m_ref_glade->get_widget("button_close", m_p_button_close);
	if(m_p_button_close)
	{
		m_p_button_close->signal_clicked().connect(sigc::mem_fun(*this, &c_preferences_dialog::hide));
	}// end if

	m_ref_glade->get_widget("button_apply", m_p_button_apply);
	if(m_p_button_apply)
	{
		m_p_button_apply->signal_clicked().connect(sigc::mem_fun(*this, &c_preferences_dialog::on_button_apply_click));
	}// end if

	m_ref_glade->get_widget("button_add_dir", m_p_button_add_dir);
	if(m_p_button_add_dir)
	{
		m_p_button_add_dir->signal_clicked().connect(sigc::mem_fun(*this, &c_preferences_dialog::on_button_add_dir_click));
	}// end if

	m_ref_glade->get_widget("button_rem_dir", m_p_button_rem_dir);
	if(m_p_button_rem_dir)
	{
		m_p_button_rem_dir->signal_clicked().connect(sigc::mem_fun(*this, &c_preferences_dialog::on_button_rem_dir_click));
	}// end if

	m_ref_glade->get_widget("entry_time", m_p_entry_time);
	m_ref_glade->get_widget("combo-entry_mode", m_p_entry_mode);
	m_ref_glade->get_widget("combo-entry_selection_mode", m_p_entry_selection_mode);
	m_ref_glade->get_widget("checkbutton_timeout", m_p_checkbutton_timeout);
	m_ref_glade->get_widget("checkbutton_logon_new", m_p_checkbutton_logon_new);
	m_ref_glade->get_widget("checkbutton_image_check", m_p_checkbutton_image_check);
	m_ref_glade->get_widget("checkbutton_follow_links", m_p_checkbutton_follow_links);
	
	// grab the tree view
	m_ref_glade->get_widget("treeview_directories", m_p_treeview_directories);

	if(m_p_treeview_directories == NULL)
	{
		throw std::runtime_error("c_preferences_dialog::c_preferences_dialog() : failed to get widget from glade");
	}// end if

	// set up the list store and model etc
	m_ref_liststore_directories = Gtk::ListStore::create(m_preferences_dialog_model_columns);

	m_p_treeview_directories->set_model(m_ref_liststore_directories);

	m_p_treeview_directories->append_column(_("Directories"), m_preferences_dialog_model_columns.directory_name);

	// update the members/cache
	c_preferences_dialog::get_settings_in_gconf();

	// populate the list
	for(std::list<Glib::ustring>::iterator it = m_ls_wp_dir.begin(); it != m_ls_wp_dir.end(); ++ it)
	{
		Gtk::TreeModel::iterator it_new_row = m_ref_liststore_directories->append();

		(*it_new_row)[m_preferences_dialog_model_columns.directory_name] = (*it);
	}// end for

	// set the other widgets up from the gconf values
	std::ostringstream ss;
	ss.imbue(std::locale("")); // use the user's locale for this stream
	ss << m_n_timeout;
	m_p_entry_time->set_text(Glib::locale_to_utf8(ss.str()));

	m_p_entry_mode->set_text(m_sz_picture_options);
	//TODO: m_p_entry_selection_mode->set_text();
	m_p_checkbutton_timeout->set_active(m_b_timeout);
	m_p_checkbutton_logon_new->set_active(m_b_wp_logon);
	m_p_checkbutton_image_check->set_active(m_b_img_check);
	m_p_checkbutton_follow_links->set_active(m_b_follow_links);

	// give the window an icon
	set_icon_from_file(GNOMEICONDIR"/wp_tray-applet.png");

	// make the c_preferences_dialog window start hidden
	c_preferences_dialog::hide();
}// end c_preferences_dialog constructor

c_preferences_dialog::~c_preferences_dialog	()
{
}// end c_preferences_dialog destructor

void
c_preferences_dialog::on_button_close_click	()
{
	c_preferences_dialog::hide();
}// end c_preferences_dialog::on_button_close_click

void
c_preferences_dialog::on_button_apply_click	()
{
	c_preferences_dialog::set_settings_in_gconf();

	c_preferences_dialog::hide();
}// end c_preferences_dialog::on_button_close_click

void
c_preferences_dialog::on_button_add_dir_click	()
{
	Gtk::FileChooserDialog gtk_filechooser_dialog(_("Wallpaper Tray - Please choose a folder"), Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);

	gtk_filechooser_dialog.set_transient_for(*this);
		
	// add response buttons the the dialog
	gtk_filechooser_dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	gtk_filechooser_dialog.add_button(_("Select"), Gtk::RESPONSE_OK);

	gtk_filechooser_dialog.set_current_folder("~");
	
	int result = gtk_filechooser_dialog.run();

	// handle the response
	switch(result)
	{
		case(Gtk::RESPONSE_OK):
		{
			m_ls_wp_dir.push_back(gtk_filechooser_dialog.get_filename());

			Gtk::TreeModel::iterator it_new_row = m_ref_liststore_directories->append();

			(*it_new_row)[m_preferences_dialog_model_columns.directory_name] = gtk_filechooser_dialog.get_filename();
		}// end case
		break;
		case(Gtk::RESPONSE_CANCEL):
		{
			// who cares?
		}// end case
		break;
		default:
		{
			// gah, i dunno, throw something?
		}// end default
		break;
	}// end switch
}// end c_preferences_dialog::on_button_close_click

void
c_preferences_dialog::on_button_rem_dir_click	()
{
	Glib::RefPtr<Gtk::TreeSelection> refptr_tree_selection = m_p_treeview_directories->get_selection();

	Gtk::TreeModel::iterator it = refptr_tree_selection->get_selected();
	
	// if anything is selected
	if(it)
	{
		Glib::ustring sz_value;

		it->get_value(0, sz_value);

		m_ls_wp_dir.erase(std::find(m_ls_wp_dir.begin(), m_ls_wp_dir.end(), sz_value));

		m_ref_liststore_directories->erase(it);
	}// end if
}// end c_preferences_dialog::on_button_close_click

void
c_preferences_dialog::get_settings_in_gconf	()
{
	m_ls_wp_dir		= m_ref_gconf_client->get_string_list("/apps/wp_tray/dir_list");
	m_n_timeout		= m_ref_gconf_client->get_int("/apps/wp_tray/n_timeout");
	m_b_timeout		= m_ref_gconf_client->get_bool("/apps/wp_tray/b_timeout");
	m_b_wp_logon		= m_ref_gconf_client->get_bool("/apps/wp_tray/b_wp_logon");
	m_b_img_check		= m_ref_gconf_client->get_bool("/apps/wp_tray/b_img_check");
	m_b_follow_links	= m_ref_gconf_client->get_bool("/apps/wp_tray/b_follow_links");
	m_sz_picture_options 	= m_ref_gconf_client->get_string("/desktop/gnome/background/picture_options");
}// end c_preferences_dialog::get_settings_from_gconf

void
c_preferences_dialog::set_settings_in_gconf	()
{
	// mwahaha scum bag C coders, we have overloading!
	m_ref_gconf_client->set_string_list("/apps/wp_tray/dir_list", m_ls_wp_dir);

	// set the other widgets up from the gconf values
	std::stringstream ss;
	ss.imbue(std::locale("")); // use the user's locale for this stream
	ss << m_p_entry_time->get_text().c_str();
	ss >> m_n_timeout;

	m_ref_gconf_client->set("/apps/wp_tray/n_timeout", m_n_timeout);
	m_ref_gconf_client->set("/apps/wp_tray/b_timeout", m_p_checkbutton_timeout->get_active());
	m_ref_gconf_client->set("/apps/wp_tray/b_wp_logon", m_p_checkbutton_logon_new->get_active());
	m_ref_gconf_client->set("/apps/wp_tray/b_img_check", m_p_checkbutton_image_check->get_active());
	m_ref_gconf_client->set("/apps/wp_tray/b_follow_links", m_p_checkbutton_follow_links->get_active());
	m_ref_gconf_client->set("/desktop/gnome/background/picture_options", m_p_entry_mode->get_text());
}// end c_preferences_dialog::set_settings_to_gconf
